@@ -1,18 +1,28 @@
+import importlib.util
 import os
-import shutil
 import platform
-import importlib
+import shutil
 import subprocess
 from pathlib import Path
-from .cmake_extension import CMakeExtension
+
 from setuptools.command.build_ext import build_ext
-from .build_ext_option import add_new_build_ext_option, BuildExtOption
 
+from .build_ext_option import BuildExtOption, add_new_build_ext_option
+from .cmake_extension import CMakeExtension
 
+# These options are listed in `python setup.py build_ext -h`
 custom_options = [
-    BuildExtOption(variable="define=", short="D",
-                   help="Create or update CMake cache "
-                        "(concatenate options with '-DBAR=b;FOO=f')")
+    BuildExtOption(
+        variable="define",
+        short="D",
+        help="Create or update CMake cache "
+        "(concatenate options with '-DBAR=b;FOO=f')",
+    ),
+    BuildExtOption(
+        variable="component",
+        short="C",
+        help="Install only a specific CMake component " "('-Cpython')",
+    ),
 ]
 
 for o in custom_options:
@@ -30,10 +40,15 @@ class BuildExtension(build_ext):
         # Initialize base class
         build_ext.initialize_options(self)
 
-        # Override define. This is supposed to pass C preprocessor macros, but we use it
-        # to pass custom options to CMake.
+        # Initialize the '--define' custom option, overriding the pre-existing one.
+        # Originally, it was aimed to pass C preprocessor definitions, but instead we
+        # use it to pass custom configuration options to CMake.
         self.define = None
 
+        # Initialize the '--component' custom option.
+        # It overrides the content of the cmake_component option of CMakeExtension.
+        self.component = None
+
     def finalize_options(self):
 
         # Parse the custom CMake options and store them in a new attribute
@@ -110,22 +125,15 @@ class BuildExtension(build_ext):
         configure_args += ext.cmake_configure_options
 
         # CMake build arguments
-        build_args = [
-            '--config', ext.cmake_build_type
-        ]
-
-        # CMake install target
-        install_target = "install"
+        build_args = ["--config", ext.cmake_build_type]
 
         if platform.system() == "Windows":
 
-            configure_args += [
-            ]
+            configure_args += []
 
         elif platform.system() in {"Linux", "Darwin"}:
 
-            configure_args += [
-            ]
+            configure_args += []
 
         else:
             raise RuntimeError(f"Unsupported '{platform.system()}' platform")
@@ -141,22 +149,36 @@ class BuildExtension(build_ext):
         configure_args += self.cmake_defines
 
         # Get the absolute path to the build folder
-        build_folder = str(Path('.').absolute() / f"{self.build_temp}_{ext.name}")
+        build_folder = str(Path(".").absolute() / f"{self.build_temp}_{ext.name}")
 
         # Make sure that the build folder exists
         Path(build_folder).mkdir(exist_ok=True, parents=True)
 
         # 1. Compose CMake configure command
-        configure_command = \
-            ['cmake', '-S', ext.source_dir, '-B', build_folder] + configure_args
+        configure_command = [
+            "cmake",
+            "-S",
+            ext.source_dir,
+            "-B",
+            build_folder,
+        ] + configure_args
 
         # 2. Compose CMake build command
-        build_command = ['cmake', '--build', build_folder] + build_args
+        build_command = ["cmake", "--build", build_folder] + build_args
 
         # 3. Compose CMake install command
-        install_command = ['cmake', '--install', build_folder]
-        if ext.cmake_component is not None:
-            install_command.extend(['--component', ext.cmake_component])
+        install_command = ["cmake", "--install", build_folder]
+
+        # If the cmake_component option of the CMakeExtension is used, install just
+        # the specified component.
+        if self.component is None and ext.cmake_component is not None:
+            install_command.extend(["--component", ext.cmake_component])
+
+        # Instead, if the `--component` command line option is used, install just
+        # the specified component. This has higher priority than what specified in
+        # the CMakeExtension.
+        if self.component is not None:
+            install_command.extend(["--component", self.component])
 
         print("")
         print("==> Configuring:")
@@ -179,6 +201,56 @@ class BuildExtension(build_ext):
             with open(file=cmake_install_prefix / "__init__.py", mode="w") as f:
                 f.write(ext.write_top_level_init)
 
+        # Write content to the bin/__main__.py magic file to expose binaries
+        if len(ext.expose_binaries) > 0:
+            bin_dirs = {str(Path(d).parents[0]) for d in ext.expose_binaries}
+
+            import inspect
+
+            main_py = inspect.cleandoc(
+                f"""
+                from pathlib import Path
+                import subprocess
+                import sys
+
+                def main():
+
+                    binary_name = Path(sys.argv[0]).name
+                    prefix = Path(__file__).parent.parent
+                    bin_dirs = {str(bin_dirs)}
+
+                    binary_path = ""
+
+                    for dir in bin_dirs:
+                        path = prefix / Path(dir) / binary_name
+                        if path.is_file():
+                            binary_path = str(path)
+                            break
+
+                        path = Path(str(path) + ".exe")
+                        if path.is_file():
+                            binary_path = str(path)
+                            break
+
+                    if not Path(binary_path).is_file():
+                        name = binary_path if binary_path != "" else binary_name
+                        raise RuntimeError(f"Failed to find binary: {{ name }}")
+
+                    sys.argv[0] = binary_path
+
+                    result = subprocess.run(args=sys.argv, capture_output=False)
+                    exit(result.returncode)
+
+                if __name__ == "__main__" and len(sys.argv) > 1:
+                    sys.argv = sys.argv[1:]
+                    main()"""
+            )
+
+            bin_folder = cmake_install_prefix / "bin"
+            Path(bin_folder).mkdir(exist_ok=True, parents=True)
+            with open(file=bin_folder / "__main__.py", mode="w") as f:
+                f.write(main_py)
+
     @staticmethod
     def extend_cmake_prefix_path(path: str) -> None:
 
@@ -188,7 +260,8 @@ class BuildExtension(build_ext):
             raise ValueError(f"Path {abs_path} does not exist")
 
         if "CMAKE_PREFIX_PATH" in os.environ:
-            os.environ["CMAKE_PREFIX_PATH"] = \
-                f"{str(path)}:{os.environ['CMAKE_PREFIX_PATH']}"
+            os.environ[
+                "CMAKE_PREFIX_PATH"
+            ] = f"{str(path)}:{os.environ['CMAKE_PREFIX_PATH']}"
         else:
             os.environ["CMAKE_PREFIX_PATH"] = str(path)
